Stan Math Library: stan/math/prim/scal/prob/weibull_log.hpp Source File

| Stan Math Library 2.10.0 reverse mode automatic differentiation

- Main Page
- Modules
- Namespaces
- Classes
- Files

- File List
- File Members

weibull_log.hpp
Go to the documentation of this file.
1 #ifndef STAN_MATH_PRIM_SCAL_PROB_WEIBULL_LOG_HPP
2 #define STAN_MATH_PRIM_SCAL_PROB_WEIBULL_LOG_HPP
3
4 #include <stan/math/prim/scal/meta/is_constant_struct.hpp>
5 #include <stan/math/prim/scal/meta/partials_return_type.hpp>
6 #include <stan/math/prim/scal/meta/OperandsAndPartials.hpp>
7 #include <stan/math/prim/scal/err/check_consistent_sizes.hpp>
8 #include <stan/math/prim/scal/err/check_finite.hpp>
9 #include <stan/math/prim/scal/err/check_nonnegative.hpp>
10 #include <stan/math/prim/scal/err/check_not_nan.hpp>
11 #include <stan/math/prim/scal/err/check_positive_finite.hpp>
12 #include <stan/math/prim/scal/fun/multiply_log.hpp>
13 #include <stan/math/prim/scal/fun/value_of.hpp>
14 #include <stan/math/prim/scal/meta/length.hpp>
15 #include <stan/math/prim/scal/fun/constants.hpp>
16 #include <stan/math/prim/scal/meta/include_summand.hpp>
17 #include <stan/math/prim/scal/meta/VectorBuilder.hpp>
18 #include <stan/math/prim/scal/meta/VectorView.hpp>
19 #include <boost/random/weibull_distribution.hpp>
20 #include <boost/random/variate_generator.hpp>
21 #include <cmath>
22
23 namespace stan {
24
25 namespace math {
26
27 // Weibull(y|alpha, sigma) [y >= 0; alpha > 0; sigma > 0]
28 // FIXME: document
29 template <bool propto,
30 typename T_y, typename T_shape, typename T_scale>
31 typename return_type<T_y, T_shape, T_scale>::type
32 weibull_log(const T_y& y, const T_shape& alpha, const T_scale& sigma) {
33 static const char* function("stan::math::weibull_log");
34 typedef typename stan::partials_return_type<T_y, T_shape, T_scale>::type
35 T_partials_return;
36
37 using stan::math::check_positive_finite;
38 using stan::math::check_finite;
39 using stan::math::check_not_nan;
40 using stan::math::value_of;
41 using stan::math::check_consistent_sizes;
42 using stan::math::multiply_log;
43 using std::log;
44
45 // check if any vectors are zero length
46 if (!(stan::length(y)
47 && stan::length(alpha)
48 && stan::length(sigma)))
49 return 0.0;
50
51 // set up return value accumulator
52 T_partials_return logp(0.0);
53 check_finite(function, "Random variable", y);
54 check_positive_finite(function, "Shape parameter", alpha);
55 check_positive_finite(function, "Scale parameter", sigma);
56 check_consistent_sizes(function,
57 "Random variable", y,
58 "Shape parameter", alpha,
59 "Scale parameter", sigma);
60
61 // check if no variables are involved and prop-to
62 if (!include_summand<propto, T_y, T_shape, T_scale>::value)
63 return 0.0;
64
65 VectorView<const T_y> y_vec(y);
66 VectorView<const T_shape> alpha_vec(alpha);
67 VectorView<const T_scale> sigma_vec(sigma);
68 size_t N = max_size(y, alpha, sigma);
69
70 for (size_t n = 0; n < N; n++) {
71 const T_partials_return y_dbl = value_of(y_vec[n]);
72 if (y_dbl < 0)
73 return LOG_ZERO;
74 }
75
76 VectorBuilder<include_summand<propto, T_shape>::value,
77 T_partials_return, T_shape> log_alpha(length(alpha));
78 for (size_t i = 0; i < length(alpha); i++)
79 if (include_summand<propto, T_shape>::value)
80 log_alpha[i] = log(value_of(alpha_vec[i]));
81
82 VectorBuilder<include_summand<propto, T_y, T_shape>::value,
83 T_partials_return, T_y> log_y(length(y));
84 for (size_t i = 0; i < length(y); i++)
85 if (include_summand<propto, T_y, T_shape>::value)
86 log_y[i] = log(value_of(y_vec[i]));
87
88 VectorBuilder<include_summand<propto, T_shape, T_scale>::value,
89 T_partials_return, T_scale> log_sigma(length(sigma));
90 for (size_t i = 0; i < length(sigma); i++)
91 if (include_summand<propto, T_shape, T_scale>::value)
92 log_sigma[i] = log(value_of(sigma_vec[i]));
93
94 VectorBuilder<include_summand<propto, T_y, T_shape, T_scale>::value,
95 T_partials_return, T_scale> inv_sigma(length(sigma));
96 for (size_t i = 0; i < length(sigma); i++)
97 if (include_summand<propto, T_y, T_shape, T_scale>::value)
98 inv_sigma[i] = 1.0 / value_of(sigma_vec[i]);
99
100 VectorBuilder<include_summand<propto, T_y, T_shape, T_scale>::value,
101 T_partials_return, T_y, T_shape, T_scale>
102 y_div_sigma_pow_alpha(N);
103 for (size_t i = 0; i < N; i++)
104 if (include_summand<propto, T_y, T_shape, T_scale>::value) {
105 const T_partials_return y_dbl = value_of(y_vec[i]);
106 const T_partials_return alpha_dbl = value_of(alpha_vec[i]);
107 y_div_sigma_pow_alpha[i] = pow(y_dbl * inv_sigma[i], alpha_dbl);
108 }
109
110 OperandsAndPartials<T_y, T_shape, T_scale>
111 operands_and_partials(y, alpha, sigma);
112 for (size_t n = 0; n < N; n++) {
113 const T_partials_return alpha_dbl = value_of(alpha_vec[n]);
114 if (include_summand<propto, T_shape>::value)
115 logp += log_alpha[n];
116 if (include_summand<propto, T_y, T_shape>::value)
117 logp += (alpha_dbl-1.0)*log_y[n];
118 if (include_summand<propto, T_shape, T_scale>::value)
119 logp -= alpha_dbl*log_sigma[n];
120 if (include_summand<propto, T_y, T_shape, T_scale>::value)
121 logp -= y_div_sigma_pow_alpha[n];
122
123 if (!is_constant_struct<T_y>::value) {
124 const T_partials_return inv_y = 1.0 / value_of(y_vec[n]);
125 operands_and_partials.d_x1[n]
126 += (alpha_dbl-1.0) * inv_y
127 - alpha_dbl * y_div_sigma_pow_alpha[n] * inv_y;
128 }
129 if (!is_constant_struct<T_shape>::value)
130 operands_and_partials.d_x2[n]
131 += 1.0/alpha_dbl
132 + (1.0 - y_div_sigma_pow_alpha[n]) * (log_y[n] - log_sigma[n]);
133 if (!is_constant_struct<T_scale>::value)
134 operands_and_partials.d_x3[n]
135 += alpha_dbl * inv_sigma[n] * (y_div_sigma_pow_alpha[n] - 1.0);
136 }
137 return operands_and_partials.value(logp);
138 }
139
140 template <typename T_y, typename T_shape, typename T_scale>
141 inline
142 typename return_type<T_y, T_shape, T_scale>::type
143 weibull_log(const T_y& y, const T_shape& alpha, const T_scale& sigma) {
144 return weibull_log<false>(y, alpha, sigma);
145 }
146 }
147 }
148 #endif
check_not_nan.hpp
stan::math::OperandsAndPartials::d_x2
VectorView< T_return_type, false, true > d_x2
Definition: OperandsAndPartials.hpp:40
stan::math::check_not_nan
bool check_not_nan(const char *function, const char *name, const T_y &y)
Return true if y is not NaN.
Definition: check_not_nan.hpp:63
check_positive_finite.hpp
stan
Definition: log_sum_exp.hpp:8
stan::math::value_of
T value_of(const fvar< T > &v)
Return the value of the specified variable.
Definition: value_of.hpp:16
stan::math::log
fvar< T > log(const fvar< T > &x)
Definition: log.hpp:15
stan::math::OperandsAndPartials::value
T_return_type value(double value)
Returns a T_return_type with the value specified with the partial derivatves.
Definition: OperandsAndPartials.hpp:68
stan::length
size_t length(const std::vector< T > &x)
Definition: length.hpp:10
is_constant_struct.hpp
stan::math::LOG_ZERO
const double LOG_ZERO
Definition: constants.hpp:175
stan::math::include_summand
Template metaprogram to calculate whether a summand needs to be included in a proportional (log) prob...
Definition: include_summand.hpp:36
stan::return_type::type
boost::math::tools::promote_args< typename scalar_type< T1 >::type, typename scalar_type< T2 >::type, typename scalar_type< T3 >::type, typename scalar_type< T4 >::type, typename scalar_type< T5 >::type, typename scalar_type< T6 >::type >::type type
Definition: return_type.hpp:27
include_summand.hpp
multiply_log.hpp
check_nonnegative.hpp
stan::is_constant_struct
Metaprogram to determine if a type has a base scalar type that can be assigned to type double...
Definition: is_constant_struct.hpp:13
stan::math::OperandsAndPartials
This class builds partial derivatives with respect to a set of operands.
Definition: OperandsAndPartials.hpp:38
VectorView.hpp
stan::math::OperandsAndPartials::d_x3
VectorView< T_return_type, false, true > d_x3
Definition: OperandsAndPartials.hpp:41
stan::math::weibull_log
return_type< T_y, T_shape, T_scale >::type weibull_log(const T_y &y, const T_shape &alpha, const T_scale &sigma)
Definition: weibull_log.hpp:32
stan::max_size
size_t max_size(const T1 &x1, const T2 &x2)
Definition: max_size.hpp:9
VectorBuilder.hpp
partials_return_type.hpp
OperandsAndPartials.hpp
length.hpp
stan::math::check_finite
bool check_finite(const char *function, const char *name, const T_y &y)
Return true if y is finite.
Definition: check_finite.hpp:62
stan::math::multiply_log
fvar< T > multiply_log(const fvar< T > &x1, const fvar< T > &x2)
Definition: multiply_log.hpp:15
stan::VectorBuilder
VectorBuilder allocates type T1 values to be used as intermediate values.
Definition: VectorBuilder.hpp:28
constants.hpp
stan::math::check_consistent_sizes
bool check_consistent_sizes(const char *function, const char *name1, const T1 &x1, const char *name2, const T2 &x2)
Return true if the dimension of x1 is consistent with x2.
Definition: check_consistent_sizes.hpp:31
check_finite.hpp
stan::math::pow
fvar< T > pow(const fvar< T > &x1, const fvar< T > &x2)
Definition: pow.hpp:18
value_of.hpp
stan::VectorView
VectorView is a template expression that is constructed with a container or scalar, which it then allows to be used as an array using operator[].
Definition: VectorView.hpp:48
stan::partials_return_type::type
boost::math::tools::promote_args< typename partials_type< typename scalar_type< T1 >::type >::type, typename partials_type< typename scalar_type< T2 >::type >::type, typename partials_type< typename scalar_type< T3 >::type >::type, typename partials_type< typename scalar_type< T4 >::type >::type, typename partials_type< typename scalar_type< T5 >::type >::type, typename partials_type< typename scalar_type< T6 >::type >::type >::type type
Definition: partials_return_type.hpp:26
check_consistent_sizes.hpp
stan::math::check_positive_finite
bool check_positive_finite(const char *function, const char *name, const T_y &y)
Return true if y is positive and finite.
Definition: check_positive_finite.hpp:28
stan::math::OperandsAndPartials::d_x1
VectorView< T_return_type, false, true > d_x1
Definition: OperandsAndPartials.hpp:39

[ Stan Home Page ] © 2011–2016, Stan Development Team.